@@ -22,7 +22,7 @@ import Data.Data
 import Text.ParserCombinators.Parsec hiding (Line)
 import Data.Set (Set)
 import qualified Data.Set as Set
-import Data.Maybe (mapMaybe)
+import Data.Maybe (mapMaybe, fromMaybe)
 
 data Result v = Error String | Ok v
     deriving (Show, Eq, Read, Data, Typeable)
@@ -42,6 +42,7 @@ data Content = ContentRaw String
              | ContentUrl Bool Deref -- ^ bool: does it include params?
              | ContentEmbed Deref
              | ContentMsg Deref
+             | ContentAttrs Deref
     deriving (Show, Eq, Read, Data, Typeable)
 
 data Line = LineForall Deref Binding
@@ -55,9 +56,10 @@ data Line = LineForall Deref Binding
           | LineOf [Ident]
           | LineTag
             { _lineTagName :: String
-            , _lineAttr :: [(Maybe Deref, String, [Content])]
+            , _lineAttr :: [(Maybe Deref, String, Maybe [Content])]
             , _lineContent :: [Content]
             , _lineClasses :: [(Maybe Deref, [Content])]
+            , _lineAttrs :: [Deref]
             }
           | LineContent [Content]
     deriving (Eq, Show, Read)
@@ -113,7 +115,7 @@ parseLine set = do
 
     invalidDollar = do
         _ <- char '$'
-        fail $ "Received a command I did not understand. If you wanted a literal $, start the line with a backslash."
+        fail "Received a command I did not understand. If you wanted a literal $, start the line with a backslash."
     comment = do
         _ <- try $ string "$#"
         _ <- many $ noneOf "\r\n"
@@ -236,14 +238,21 @@ parseLine set = do
         tagClass (Just d) <|> tagAttrib (Just d)
     tagClass x = char '.' >> (TagClass . (,) x) <$> tagAttribValue NotInQuotes
     tagAttrib cond = do
-        s <- many1 $ noneOf " \t=\r\n>"
-        v <- (char '=' >> tagAttribValue NotInQuotesAttr) <|> return []
+        s <- many1 $ noneOf " \t=\r\n><"
+        v <- (char '=' >> Just <$> tagAttribValue NotInQuotesAttr) <|> return Nothing
         return $ TagAttrib (cond, s, v)
-    tag' = foldr tag'' ("div", [], [])
-    tag'' (TagName s) (_, y, z) = (s, y, z)
-    tag'' (TagIdent s) (x, y, z) = (x, (Nothing, "id", s) : y, z)
-    tag'' (TagClass s) (x, y, z) = (x, y, s : z)
-    tag'' (TagAttrib s) (x, y, z) = (x, s : y, z)
+
+    tagAttrs = do
+        _ <- char '*'
+        d <- between (char '{') (char '}') parseDeref
+        return $ TagAttribs d
+
+    tag' = foldr tag'' ("div", [], [], [])
+    tag'' (TagName s) (_, y, z, as) = (s, y, z, as)
+    tag'' (TagIdent s) (x, y, z, as) = (x, (Nothing, "id", Just s) : y, z, as)
+    tag'' (TagClass s) (x, y, z, as) = (x, y, s : z, as)
+    tag'' (TagAttrib s) (x, y, z, as) = (x, s : y, z, as)
+    tag'' (TagAttribs s) (x, y, z, as) = (x, y, z, s : as)
 
     ident :: Parser Ident
     ident = Ident <$> many1 (alphaNum <|> char '_' <|> char '\'')
@@ -264,18 +273,21 @@ parseLine set = do
         _ <- char '<'
         name' <- many  $ noneOf " \t.#\r\n!>"
         let name = if null name' then "div" else name'
-        xs <- many $ try ((many $ oneOf " \t") >>
-              (tagIdent <|> tagCond <|> tagClass Nothing <|> tagAttrib Nothing))
-        _ <- many $ oneOf " \t"
-        c <- (eol >> return []) <|> (char '>' >> content InContent)
-        let (tn, attr, classes) = tag' $ TagName name : xs
-        when ('/' `elem` tn) $ error "A tag name may not contain a slash"
-        return $ LineTag tn attr c classes
+        xs <- many $ try ((many $ oneOf " \t\r\n") >>
+              (tagIdent <|> tagCond <|> tagClass Nothing <|> tagAttrs <|> tagAttrib Nothing))
+        _ <- many $ oneOf " \t\r\n"
+        _ <- char '>'
+        c <- content InContent
+        let (tn, attr, classes, attrsd) = tag' $ TagName name : xs
+        if '/' `elem` tn
+          then fail "A tag name may not contain a slash. Perhaps you have a closing tag in your HTML."
+          else return $ LineTag tn attr c classes attrsd
 
 data TagPiece = TagName String
               | TagIdent [Content]
               | TagClass (Maybe Deref, [Content])
-              | TagAttrib (Maybe Deref, String, [Content])
+              | TagAttrib (Maybe Deref, String, Maybe [Content])
+              | TagAttribs Deref
     deriving Show
 
 data ContentRule = InQuotes | NotInQuotes | NotInQuotesAttr | InContent
@@ -329,9 +341,9 @@ nestToDoc set (Nest (LineCase d) inside:rest) = do
     cases <- mapM getOf inside
     rest' <- nestToDoc set rest
     Ok $ DocCase d cases : rest'
-nestToDoc set (Nest (LineTag tn attrs content classes) inside:rest) = do
+nestToDoc set (Nest (LineTag tn attrs content classes attrsD) inside:rest) = do
     let attrFix (x, y, z) = (x, y, [(Nothing, z)])
-    let takeClass (a, "class", b) = Just (a, b)
+    let takeClass (a, "class", b) = Just (a, fromMaybe [] b)
         takeClass _ = Nothing
     let clazzes = classes ++ mapMaybe takeClass attrs
     let notClass (_, x, _) = x /= "class"
@@ -339,7 +351,7 @@ nestToDoc set (Nest (LineTag tn attrs content classes) inside:rest) = do
     let attrs' =
             case clazzes of
               [] -> map attrFix noclass
-              _ -> (Nothing, "class", clazzes)
+              _ -> (Nothing, "class", map (second Just) clazzes)
                        : map attrFix noclass
     let closeStyle =
             if not (null content) || not (null inside)
@@ -355,12 +367,14 @@ nestToDoc set (Nest (LineTag tn attrs content classes) inside:rest) = do
         start = DocContent $ ContentRaw $ "<" ++ tn
         attrs'' = concatMap attrToContent attrs'
         newline' = DocContent $ ContentRaw
-                 $ if hamletCloseNewline set then "\n" else ""
+                 $ if hamletNewlines set then "\n" else ""
     inside' <- nestToDoc set inside
     rest' <- nestToDoc set rest
     Ok $ start
        : attrs''
+      ++ map (DocContent . ContentAttrs) attrsD
       ++ seal
+       : newline'
        : map DocContent content
       ++ inside'
       ++ end
@@ -369,7 +383,9 @@ nestToDoc set (Nest (LineTag tn attrs content classes) inside:rest) = do
 nestToDoc set (Nest (LineContent content) inside:rest) = do
     inside' <- nestToDoc set inside
     rest' <- nestToDoc set rest
-    Ok $ map DocContent content ++ inside' ++ rest'
+    let newline' = DocContent $ ContentRaw
+                   $ if hamletNewlines set then "\n" else ""
+    Ok $ map DocContent content ++ newline':inside' ++ rest'
 nestToDoc _set (Nest (LineElseIf _) _:_) = Error "Unexpected elseif"
 nestToDoc _set (Nest LineElse _:_) = Error "Unexpected else"
 nestToDoc _set (Nest LineNothing _:_) = Error "Unexpected nothing"
@@ -407,12 +423,12 @@ parseDoc set s = do
     ds <- nestToDoc set ns
     return $ compressDoc ds
 
-attrToContent :: (Maybe Deref, String, [(Maybe Deref, [Content])]) -> [Doc]
+attrToContent :: (Maybe Deref, String, [(Maybe Deref, Maybe [Content])]) -> [Doc]
 attrToContent (Just cond, k, v) =
     [DocCond [(cond, attrToContent (Nothing, k, v))] Nothing]
 attrToContent (Nothing, k, []) = [DocContent $ ContentRaw $ ' ' : k]
-attrToContent (Nothing, k, [(Nothing, [])]) = [DocContent $ ContentRaw $ ' ' : k]
-attrToContent (Nothing, k, [(Nothing, v)]) =
+attrToContent (Nothing, k, [(Nothing, Nothing)]) = [DocContent $ ContentRaw $ ' ' : k]
+attrToContent (Nothing, k, [(Nothing, Just v)]) =
     DocContent (ContentRaw (' ' : k ++ "=\""))
   : map DocContent v
   ++ [DocContent $ ContentRaw "\""]
@@ -422,16 +438,16 @@ attrToContent (Nothing, k, v) = -- only for class
     ++ go' (last v)
     ++ [DocContent $ ContentRaw "\""]
   where
-    go (Nothing, x) = map DocContent x ++ [DocContent $ ContentRaw " "]
+    go (Nothing, x) = map DocContent (fromMaybe [] x) ++ [DocContent $ ContentRaw " "]
     go (Just b, x) =
         [ DocCond
-            [(b, map DocContent x ++ [DocContent $ ContentRaw " "])]
+            [(b, map DocContent (fromMaybe [] x) ++ [DocContent $ ContentRaw " "])]
             Nothing
         ]
-    go' (Nothing, x) = map DocContent x
+    go' (Nothing, x) = maybe [] (map DocContent) x
     go' (Just b, x) =
         [ DocCond
-            [(b, map DocContent x)]
+            [(b, maybe [] (map DocContent) x)]
             Nothing
         ]
 
@@ -441,9 +457,9 @@ data HamletSettings = HamletSettings
       -- | The value to replace a \"!!!\" with. Do not include the trailing
       -- newline.
       hamletDoctype :: String
-      -- | Should we put a newline after closing a tag? Mostly useful for debug
-      -- output.
-    , hamletCloseNewline :: Bool
+      -- | Should we add newlines to the output, making it more human-readable?
+      --  Useful for client-side debugging but may alter browser page layout.
+    , hamletNewlines :: Bool
       -- | How a tag should be closed. Use this to switch between HTML, XHTML
       -- or even XML output.
     , hamletCloseStyle :: String -> CloseStyle
