@@ -2,4 +2,4 @@ import HamletTest (specs)
 import Test.Hspec
 
 main :: IO ()
-main = hspecX $ descriptions [specs]
+main = hspecX [specs]
