@@ -16,8 +16,9 @@ import qualified Data.Map as Map
 import Data.Text (Text, pack, unpack)
 import Data.Monoid (mappend)
 import qualified Data.Set as Set
-import qualified Text.Blaze.Renderer.Text
-import Text.Blaze (toHtml, preEscapedString)
+import qualified Text.Blaze.Html.Renderer.Text
+import Text.Blaze.Html (toHtml)
+import Text.Blaze.Internal (preEscapedString)
 
 specs = describe "hamlet"
   [ it "empty" caseEmpty
@@ -83,7 +84,7 @@ $#a third one|]
 
   , it "ignores a blank line" $ do
     helper "<p>foo</p>" [hamlet|
-<p
+<p>
 
     foo
 
@@ -96,7 +97,7 @@ $#a third one|]
   , it "hamlet angle bracket syntax" $
       helper "<p class=\"foo\" height=\"100\"><span id=\"bar\" width=\"50\">HELLO</span></p>"
         [hamlet|
-<p.foo height="100"
+<p.foo height="100">
     <span #bar width=50>HELLO
 |]
 
@@ -132,6 +133,14 @@ $#a third one|]
   , it "embedded slash" $ do
     helper "///" [hamlet|///|]
 
+{- compile-time error
+  , it "tag with slash" $ do
+    helper "" [hamlet|
+<p>
+  Text
+</p>
+|]
+-}
 
   , it "string literals" $ do
     helper "string" [hamlet|#{"string"}|]
@@ -150,7 +159,7 @@ $#a third one|]
     helper "<p>1</p><p>2 not ignored</p>" [hamlet|
 <p>1
 <!-- ignored comment -->
-<p
+<p>
     2
     <!-- ignored --> not ignored<!-- ignored -->
 |]
@@ -184,13 +193,13 @@ $nothing
 
   , it "conditional class" $ do
       helper "<p class=\"current\"></p>" 
-        [hamlet|<p :False:.ignored :True:.current|]
+        [hamlet|<p :False:.ignored :True:.current>|]
 
       helper "<p class=\"1 3 2 4\"></p>"
-        [hamlet|<p :True:.1 :True:class=2 :False:.a :False:class=b .3 class=4|]
+        [hamlet|<p :True:.1 :True:class=2 :False:.a :False:class=b .3 class=4>|]
 
       helper "<p class=\"foo bar baz\"></p>"
-        [hamlet|<p class=foo class=bar class=baz|]
+        [hamlet|<p class=foo class=bar class=baz>|]
 
 
 
@@ -257,10 +266,12 @@ $forall x <- set
     $forall num <- [1, 2, 3]
         <li>#{show num}
 |]
+{-
   , it "infix operators" $
       helper "5" [hamlet|#{show $ (4 + 5) - (2 + 2)}|]
   , it "infix operators with parens" $
       helper "5" [hamlet|#{show (2 + 3)}|]
+      -}
   , it "doctypes" $ helper "<!DOCTYPE html>\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n" [hamlet|
 $doctype 5
 $doctype strict
@@ -330,6 +341,20 @@ $case url2
         <dt>#{name}
         <dd>#{show age}
 |]
+
+  , it "multiline tags" $ helper
+      "<foo bar=\"baz\" bin=\"bin\">content</foo>" [hamlet|
+<foo bar=baz
+     bin=bin>content
+|]
+  , let attrs = [("bar", "baz"), ("bin", "<>\"&")]
+     in it "*{...} attributes" $ helper
+      "<foo bar=\"baz\" bin=\"&lt;&gt;&quot;&amp;\">content</foo>" [hamlet|
+<foo *{attrs}>content
+|]
+  , it "blank attr values" $ helper
+      "<foo bar=\"\" baz bin=\"\"></foo>"
+      [hamlet|<foo bar="" baz bin=>|]
   ]
 
 data Pair = Pair String Int
@@ -400,12 +425,12 @@ theArg = Arg
 
 helperHtml :: String -> Html -> Assertion
 helperHtml res h = do
-    let x = Text.Blaze.Renderer.Text.renderHtml h
+    let x = Text.Blaze.Html.Renderer.Text.renderHtml h
     T.pack res @=? x
 
 helper :: String -> HtmlUrl Url -> Assertion
 helper res h = do
-    let x = Text.Blaze.Renderer.Text.renderHtml $ h render
+    let x = Text.Blaze.Html.Renderer.Text.renderHtml $ h render
     T.pack res @=? x
 
 caseEmpty :: Assertion
@@ -417,11 +442,11 @@ caseStatic = helper "some static content" [hamlet|some static content|]
 caseTag :: Assertion
 caseTag = do
     helper "<p class=\"foo\"><div id=\"bar\">baz</div></p>" [hamlet|
-<p .foo
+<p .foo>
   <#bar>baz
 |]
     helper "<p class=\"foo.bar\"><div id=\"bar\">baz</div></p>" [hamlet|
-<p class=foo.bar
+<p class=foo.bar>
   <#bar>baz
 |]
 
@@ -552,30 +577,23 @@ $with n <- " , something", y <- n
 |]
 
 caseScriptNotEmpty :: Assertion
-caseScriptNotEmpty = helper "<script></script>" [hamlet|<script|]
+caseScriptNotEmpty = helper "<script></script>" [hamlet|<script>|]
 
 caseMetaEmpty :: Assertion
 caseMetaEmpty = do
-    helper "<meta>" [hamlet|<meta|]
-    helper "<meta/>" [xhamlet|<meta|]
     helper "<meta>" [hamlet|<meta>|]
     helper "<meta/>" [xhamlet|<meta>|]
 
 caseInputEmpty :: Assertion
 caseInputEmpty = do
-    helper "<input>" [hamlet|<input|]
-    helper "<input/>" [xhamlet|<input|]
     helper "<input>" [hamlet|<input>|]
     helper "<input/>" [xhamlet|<input>|]
 
 caseMultiClass :: Assertion
-caseMultiClass = do
-    helper "<div class=\"foo bar\"></div>" [hamlet|<.foo.bar|]
-    helper "<div class=\"foo bar\"></div>" [hamlet|<.foo.bar>|]
+caseMultiClass = helper "<div class=\"foo bar\"></div>" [hamlet|<.foo.bar>|]
 
 caseAttribOrder :: Assertion
-caseAttribOrder = do
-    helper "<meta 1 2 3>" [hamlet|<meta 1 2 3|]
+caseAttribOrder =
     helper "<meta 1 2 3>" [hamlet|<meta 1 2 3>|]
 
 caseNothing :: Assertion
@@ -638,11 +656,11 @@ caseEmptyStatementList = do
 
 caseAttribCond :: Assertion
 caseAttribCond = do
-    helper "<select></select>" [hamlet|<select :False:selected|]
-    helper "<select selected></select>" [hamlet|<select :True:selected|]
-    helper "<meta var=\"foo:bar\">" [hamlet|<meta var=foo:bar|]
+    helper "<select></select>" [hamlet|<select :False:selected>|]
+    helper "<select selected></select>" [hamlet|<select :True:selected>|]
+    helper "<meta var=\"foo:bar\">" [hamlet|<meta var=foo:bar>|]
     helper "<select selected></select>"
-        [hamlet|<select :true theArg:selected|]
+        [hamlet|<select :true theArg:selected>|]
 
     helper "<select></select>" [hamlet|<select :False:selected>|]
     helper "<select selected></select>" [hamlet|<select :True:selected>|]
@@ -677,19 +695,19 @@ caseNonLeadingPercent =
 caseQuotedAttribs :: Assertion
 caseQuotedAttribs =
     helper "<input type=\"submit\" value=\"Submit response\">" [hamlet|
-<input type=submit value="Submit response"
+<input type=submit value="Submit response">
 |]
 
 caseSpacedDerefs :: Assertion
 caseSpacedDerefs = do
     helper "&lt;var&gt;" [hamlet|#{var theArg}|]
-    helper "<div class=\"&lt;var&gt;\"></div>" [hamlet|<.#{var theArg}|]
+    helper "<div class=\"&lt;var&gt;\"></div>" [hamlet|<.#{var theArg}>|]
 
 caseAttribVars :: Assertion
 caseAttribVars = do
-    helper "<div id=\"&lt;var&gt;\"></div>" [hamlet|<##{var theArg}|]
-    helper "<div class=\"&lt;var&gt;\"></div>" [hamlet|<.#{var theArg}|]
-    helper "<div f=\"&lt;var&gt;\"></div>" [hamlet|< f=#{var theArg}|]
+    helper "<div id=\"&lt;var&gt;\"></div>" [hamlet|<##{var theArg}>|]
+    helper "<div class=\"&lt;var&gt;\"></div>" [hamlet|<.#{var theArg}>|]
+    helper "<div f=\"&lt;var&gt;\"></div>" [hamlet|< f=#{var theArg}>|]
 
     helper "<div id=\"&lt;var&gt;\"></div>" [hamlet|<##{var theArg}>|]
     helper "<div class=\"&lt;var&gt;\"></div>" [hamlet|<.#{var theArg}>|]
@@ -706,10 +724,10 @@ caseNesting = do
     helper
       "<table><tbody><tr><td>1</td></tr><tr><td>2</td></tr></tbody></table>"
       [hamlet|
-<table
-  <tbody
+<table>
+  <tbody>
     $forall user <- users
-        <tr
+        <tr>
          <td>#{user}
 |]
     helper
@@ -720,8 +738,8 @@ caseNesting = do
           , "</select>"
           ])
         [hamlet|
-<select #"#{name}" name=#{name}
-    <option :isBoolBlank val:selected
+<select #"#{name}" name=#{name}>
+    <option :isBoolBlank val:selected>
     <option value=true :isBoolTrue val:selected>Yes
     <option value=false :isBoolFalse val:selected>No
 |]
@@ -771,20 +789,20 @@ caseHamletLiterals = do
     helper "-123.456" [hamlet|#{show -123.456}|]
 
 helper' :: String -> Html -> Assertion
-helper' res h = T.pack res @=? Text.Blaze.Renderer.Text.renderHtml h
+helper' res h = T.pack res @=? Text.Blaze.Html.Renderer.Text.renderHtml h
 
 caseHamlet' :: Assertion
 caseHamlet' = do
     helper' "foo" [shamlet|foo|]
     helper' "foo" [xshamlet|foo|]
-    helper "<br>" $ const $ [shamlet|<br|]
-    helper "<br/>" $ const $ [xshamlet|<br|]
+    helper "<br>" $ const $ [shamlet|<br>|]
+    helper "<br/>" $ const $ [xshamlet|<br>|]
 
     -- new with generalized stuff
     helper' "foo" [shamlet|foo|]
     helper' "foo" [xshamlet|foo|]
-    helper "<br>" $ const $ [shamlet|<br|]
-    helper "<br/>" $ const $ [xshamlet|<br|]
+    helper "<br>" $ const $ [shamlet|<br>|]
+    helper "<br/>" $ const $ [xshamlet|<br>|]
 
 
 instance Show Url where
@@ -793,7 +811,7 @@ instance Show Url where
 caseDiffBindNames :: Assertion
 caseDiffBindNames = do
     let list = words "1 2 3"
-    -- FIXME helper "123123" $(hamletFileDebug "test/hamlets/external-debug3.hamlet")
+    -- FIXME helper "123123" $(hamletFileReload "test/hamlets/external-debug3.hamlet")
     error "test has been disabled"
 
 
@@ -816,7 +834,7 @@ data Msg = Hello | Goodbye
 
 ihelper :: String -> HtmlUrlI18n Msg Url -> Assertion
 ihelper res h = do
-    let x = Text.Blaze.Renderer.Text.renderHtml $ h showMsg render
+    let x = Text.Blaze.Html.Renderer.Text.renderHtml $ h showMsg render
     T.pack res @=? x
   where
     showMsg Hello = preEscapedString "Hola"
